%Assumptions: The only forces present are the ones from gravity.
%All gravel in the bucket will be placed as in figLoad1.fig. Gravel
%that is considered in the bucket is the gravel on the same side as
%the bucket of the line through the rightmost end point of the
%bucket and perpendicular to the gound.
function load1

bucketRearAngle = pi/3;
bucketSideLength = 1.5; %m
%bucketDepth = 3; %m
bucketMass = 1e3; %kg
%bucketVolume = bucketSideLength^2*sin(bucketRearAngle)/2*bucketDepth;
bucketVolume = bucketSideLength^2*sin(bucketRearAngle)/2;
%bucketTiltAngle = pi/5;

gravelDensity = 2e3; %kg/m^3
gravelAngle = pi/4;

yalmipsetup
sedumisetup

g = 9.81;
yMax = 4;
energy = 0;
tt = @centGravBucketAndGravel;
%Potential energy for an empty bucket when at ground:
[x y] = tt(bucketSideLength, bucketRearAngle, bucketMass, ...
	   0, gravelDensity, bucketRearAngle/2);
energy = energy - g*bucketMass*y;
%Potential energy for a full bucket when at end position:
[x y] = tt(bucketSideLength, bucketRearAngle, bucketMass, ...
	   bucketVolume, gravelDensity, bucketRearAngle/2);
add = g*(bucketMass+gravelDensity*bucketVolume)* ...
      (yMax-bucketSideLength*cos(bucketRearAngle/2)+y);
energy = energy + add;
%Potential energy for the unmoved gravel:
N = 10;
y = linspace(0, yMax, N)';
Y = y(2:end-1);
dy = yMax/(N-1);
x = sdpvar(N-2, 1);
X = x-Y/tan(gravelAngle);
gravelVolume = sum(X)*dy;
energy = energy - g* gravelDensity*sum(X.*Y)*dy;
%Constraints:
tmp = -eye(N-2) + diag(ones(1,N-3),1);
constr = set(Y/tan(gravelAngle) <= x) + ...
	 set(gravelVolume == bucketVolume) + ...
	 set(x(1) >= 0) + set(x(end) <= yMax/tan(gravelAngle));
if N > 3,
  constr = constr + set(tmp(1:(end-1), :)*x >= 0);
end
opts = sdpsettings;

%Compression force:
Ktot = 6e2;

%To enforce that the bottom surface of the bucket is not pushed
%through the gravel.
if 1
bucketTiltAngle = sdpvar(N-2, 1);
maxDTilt = (pi/2 - bucketRearAngle/2)/(N-1)*2;
constr = constr + set(bucketTiltAngle >= bucketRearAngle/2) + ...
	 set(bucketTiltAngle <= pi/2);
tmp =  [bucketTiltAngle; pi/2]-[bucketRearAngle/2; bucketTiltAngle];
constr = constr + set(-maxDTilt <= tmp) + set(tmp <= maxDTilt);
dx = [x(2:end); yMax]-x;
tanBeta = sdpfun(pi/2-(bucketTiltAngle-bucketRearAngle/2), 'tan');
constr = constr + set(dx/dy - tanBeta <= 0);
opts = sdpsettings('solver','bmibnb');
end

solvesdp(constr, energy, opts)

x = [0; double(x); yMax/tan(gravelAngle)];
energy = double(energy)

plot(y/tan(gravelAngle), y, 'k', x, y, 'b-.');

bucketTiltAngle = double(bucketTiltAngle);
tan(pi/2-(bucketTiltAngle-bucketRearAngle/2))
dx = [x(2:end); yMax]-x;
dx/dy
    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [x y] = centGravBucketAndGravel(bucketSideLength, ...
					 bucketRearAngle, bucketMass, ...
					 gravelVolume, gravelDensity, ...
					 bucketTiltAngle)
%The coordinates of the center of gravity when the bucket is
%upright (i.e. looking like a V)
%bucketCG_x = 0;
bucketCentGravity_y = bucketSideLength*cos(bucketRearAngle/2)/2;

%gravelCentGravity_x = 0;
gravelCentGravity_y = 2/3*sqrt(gravelVolume/tan(bucketRearAngle/2));
gravelMass = gravelVolume*gravelDensity;

tmp = (bucketCentGravity_y*bucketMass + gravelCentGravity_y*gravelMass) ...
      /(bucketMass + gravelMass);
x = tmp*cos(bucketTiltAngle);
y = tmp*sin(bucketTiltAngle);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function drawBucketAndGravel(bucketSideLength, bucketRearAngle, ...
			     gravelVolume, bucketTiltAngle)

s = bucketSideLength;
a = bucketRearAngle;
b = bucketTiltAngle;
bucketX = s*[cos(b+a/2) 0 cos(b-a/2)];
bucketY = s*[sin(b+a/2) 0 sin(b-a/2)];

gs = sqrt(2*gravelVolume/sin(a));
gravelX = gs*[cos(b+a/2) cos(b-a/2)];
gravelY = gs*[sin(b+a/2) sin(b-a/2)];

figure(1); clf;
hold on
plot(bucketX, bucketY, 'b', 'LineWidth', 2);
if gravelVolume > 0,
  plot(gravelX, gravelY, 'k');
end
grid on
axis equal